
#include "SDL.h"
#include "SDL_opengl.h"

#include "block.h"
#include "cube.h"
#include "random.h"
#include "coordinate.h"

#include "tweak/tweak.h"

namespace PRJID {

	static unsigned int stat=0;

	Block::Block() {
		active=true;
	}

	bool Block::is_splitted() {
		return sub.size()>0;
	}
	bool Block::is_active() {
		return active;
	}

	void Block::activate() {
		active=true;
	}
	void Block::deactivate() {
		active=false;
	}
	void Block::toggle() {
		active=!active;
	}

	unsigned int Block::get_stat() {
		unsigned int r=stat;
		stat=0;
		return r;
	}


	void Block::split() {
		if (is_splitted()) return;
		sub.resize(8);
	}
	void Block::merge() { sub.resize(0); }

	void Block::check() {
		if (is_splitted()) {
			for (unsigned int i=0;i<8;i++) {
				sub[i].check();
			}
			unsigned int act=0;
			unsigned int inact=0;
			for (unsigned int i=0;i<8;i++) {
				if (sub[i].is_splitted()) return;
				if (sub[i].is_active()) { act++; } else { inact++;}
			}
			if (act==8) { 
				merge(); 
				active=true; 
				return; 
			}
			if (inact==8) { 
				merge(); 
				active=false; 
				return; 
			}
			return;
		}
		return;
	}


	Block *Block::get_child(unsigned int b){
		if (!is_splitted()) return NULL;
		return &sub[b];
	}

	void Block::top_draw() {
		stat=0;
		draw(Blockaddress());
	}

	Blockaddress Block::top_select(unsigned int x,unsigned int y) {
		Blockpick pick(x,y);
		Blockaddress a;
		a.set_pick(&pick);

		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
		draw(a);
		return pick.result();
	}

	Blockaddress Block::random(Blockaddress a) {
		if (is_splitted()) {
			unsigned int r;
			std::vector <int> vactive;
			for (unsigned int i=0;i<8;i++) {
				if (!sub[i].is_active()) continue;
				vactive.push_back(i);
			}

			if (vactive.size()==0) {
				r=0;
			} else {
				r=vactive[Random::sget(vactive.size())];
			}
			return sub[r].random(a.get_descend(r,&sub[r]));
		}
		return a;
	}

	bool Block::frustum_test(Blockaddress a) {

		C3f pos;
		pos=a.get_pos();
		pos*=Tweak::blockscale_f();
		if (Frustum::instance()->test_point(pos)) { return true; }

		float s;
		s=a.get_scale();
		s*=Tweak::blockscale_f();
		C3f corner;
		float radius;
		radius=s;

		corner=pos+C3f(s,s,s);
		if (Frustum::instance()->test_point(corner,radius)) { return true; }

		corner=pos+C3f(-s,s,s);
		if (Frustum::instance()->test_point(corner,radius)) { return true; }

		corner=pos+C3f(s,s,-s);
		if (Frustum::instance()->test_point(corner,radius)) { return true; }

		corner=pos+C3f(-s,s,-s);
		if (Frustum::instance()->test_point(corner,radius)) { return true; }


		corner=pos+C3f(s,-s,s);
		if (Frustum::instance()->test_point(corner,radius)) { return true; }

		corner=pos+C3f(-s,-s,s);
		if (Frustum::instance()->test_point(corner,radius)) { return true; }

		corner=pos+C3f(s,-s,-s);
		if (Frustum::instance()->test_point(corner,radius)) { return true; }

		corner=pos+C3f(-s,-s,-s);
		if (Frustum::instance()->test_point(corner,radius)) { return true; }

		return false;
	}

	void Block::draw(Blockaddress a) {
		if (is_splitted()) {
			
			for (unsigned int i=0;i<8;i++) {
				glPushMatrix();
				glScalef(0.5,0.5,0.5);
				a.translate(i);
				sub[i].draw(a.get_descend(i,&sub[i]));
				glPopMatrix();
			}
			return;
		}
		if (Tweak::editmode_i()==1) 
		if (a.is_selected()||(!is_active()&&Tweak::draw_all_i()==1)) {
			glPushMatrix();
			if (!is_active())  { 
				float sc=Tweak::inactive_scale_f();
				float tr=Tweak::inactive_trans_f();
				glScalef(sc,sc,sc); glTranslatef(tr,tr,tr); 
			}
			if (a.is_selected()) { glColor3f(1,1,1);  } else { glColor3f(.2,.3,.4); }
			Cube::draw();
			a.touch();
			glPopMatrix();
			return;
		}
		if (!is_active()) return;

		if (frustum_test(a)) {
			stat++;
			Frustum::instance()->drawn();

			if (Tweak::coloredcubes_i()==1) {
				Cube::draw_colored();
			} else {
				glColor3f(0,0,0);
				Cube::draw();
			}
		} else {
			Frustum::instance()->notdrawn();
		}
		a.touch();
		return;
	}
	bool Block::top_does_collide(C3f p, float size) {
		Blockaddress a;
		return does_collide(p,size,a);
	}
	bool Block::does_collide(C3f p,float size,Blockaddress a) {
		if (is_splitted()) {
			for (unsigned int i=0;i<8;i++) {
				if (sub[i].does_collide(p,size,a.get_descend(i,&sub[i]))) return true;
			}
			return false;
		}
		if (!is_active()) return false;
		C3f pos;
		Box3f box;


		pos=a.get_pos();
		pos*=Tweak::blockscale_f();
		pos-=size;
		box.add(pos);

		pos=a.get_pos();
		pos+=a.scale_to_level(1);
		pos*=Tweak::blockscale_f();
		pos+=size;
		box.add(pos);
		C3f d;
		d=a.get_pos();
		if (box.iswithin(p)) return true;
		return false;
	}


	bool Block::top_save(std::string filename) {
		std::ofstream of;
		of.open(filename.c_str());
		if (!of) return false;
		save(of);
		of.close();
		return true;
	}
	bool Block::top_load(std::string filename) {
		merge();
		std::ifstream ifstr;
		ifstr.open(filename.c_str());
		if (!ifstr) return false;
		load(ifstr);
		ifstr.close();
		return true;
	}

	void Block::save(std::ofstream& st) {
		char status=0;
		if (is_splitted()) {
			status=1;
		} else {
			status=3;
			if (is_active()) {status=2; }
		}
		st << status;
		if (!is_splitted()) { return; }
		for (unsigned int i=0;i<8;i++) {
			sub[i].save(st);
		}
	}
	void Block::load(std::ifstream& st) {
		if (st.eof()) return;
		char status;
		st >> status;
		switch (status) {
			case 2: active=true; return;
			case 3: active=false; return;
			case 1: break;
			default:
				std::cout << "invalid data " << status << std::endl;
				return;
		}
		split();
		for (unsigned int i=0;i<8;i++) {
			sub[i].load(st);
		}
	}


	Blockpick::Blockpick(unsigned int nx,unsigned int ny) {
		x=nx;
		y=ny;
		depth=std::numeric_limits<unsigned int>::max();
	}

	void Blockpick::test(Blockaddress *a) {
      unsigned int p;
      glReadPixels(
         x,y,1,1,
         GL_DEPTH_COMPONENT, GL_UNSIGNED_INT,(GLvoid *)&p
      );

      if (p>=depth) return;
      depth=p;
      selected=*a;
      return;
	}

} //namespace
