
#include "SDL_opengl.h"

#include "blockaddress.h"
#include "block.h"

namespace PRJID {

Blockaddress::Blockaddress() {
	level=0;
	pick=NULL;
	current=NULL;
	parent=NULL;
}


const unsigned int levels[8]={
	10000000,
	5000000,
	2500000,
	1250000,
	625000,
	312500,
	156250,
	78125	
};

const unsigned int sub_positions[8][3]={
	{0,0,0},
	{1,0,0},
	{0,1,0},
	{1,1,0},

	{0,0,1},
	{1,0,1},
	{0,1,1},
	{1,1,1}
};

const float scales[8]={1,.5,.25,.125,0.0625,0.03125,0.015625,0.0078125};

C3f Blockaddress::get_pos() {

	C3f r;
	r=pos.tofloat();
	r/=static_cast<float>(levels[0]);
	return r;
}
C3 Blockaddress::get_int_pos() {
	return pos;
}

float Blockaddress::get_scale() {
	return scales[level];
}
C3f Blockaddress::scale_to_level(C3f p) {
	p*=scales[level];
	return p;
}
bool Blockaddress::is_selected() {
	if (pos==selected) return true;
	return false;
}

void Blockaddress::select(Blockaddress a) {
	selected=a.get_int_pos();
}

void Blockaddress::reset_pos() {
	pos=C3(-1,-1,-1);
}

void Blockaddress::touch() {
	if (pick==NULL) { return; }
	pick->test(this);
}

void Blockaddress::set_pick(Blockpick *p) {
	pick=p;
}

Block *Blockaddress::get_current() { return current; }
Block *Blockaddress::get_parent() { return parent; }

Blockaddress Blockaddress::get_descend(unsigned int b,Block *p) {
	Blockaddress a;
	a=*this;

	a.descend(b,p);
	return a;
}

void Blockaddress::descend(unsigned int b,Block *p) {
	if (b>7) {
		return;
	}
	parent=current;
	current=p;

	level++;

	unsigned int step;
	step=levels[level];
	pos.x+=step*sub_positions[b][0];
	pos.y+=step*sub_positions[b][1];
	pos.z+=step*sub_positions[b][2];
}

void Blockaddress::translate(unsigned int b) {
	glTranslatef(
		sub_positions[b][0],
		sub_positions[b][1],
		sub_positions[b][2]
	);
}



} //namespace
